import {
  watch
} from "./chunk.2WO4NHJ2.js";

// libs/core/src/utils/decorators/style-expression-property.ts
import { unsafeCSS } from "lit";
import { property } from "lit/decorators.js";

// libs/core/src/utils/helpers/style-expression-parser.ts
var viewportBreakpoints = {
  "0": "0px",
  "2xs": "320px",
  xs: "425px",
  s: "768px",
  m: "1024px",
  l: "1280px",
  xl: "1440px",
  "2xl": "2560px",
  "3xl": "3840px",
  "4xl": "4320px",
  "5xl": "6016px",
  "6xl": "7680px"
};
var breakpointValueRegex = /^([<|>]=?)?([0-9a-z]+)/;
var controlTokens = ["{", "}", ";", ":", ","];
var whitespace = [" ", "/n"];
function tokenize(source) {
  const lexemes = [];
  let scanned = "";
  for (let i = 0; i < source.length; i++) {
    const c = source[i];
    if (!whitespace.includes(c))
      scanned += c;
    if (controlTokens.includes(c)) {
      lexemes.push(scanned.slice(0, -1));
      lexemes.push(c);
      scanned = "";
      continue;
    }
    if (whitespace.includes(c) || i === source.length - 1) {
      lexemes.push(scanned);
      scanned = "";
      continue;
    }
  }
  return lexemes.filter((l) => l !== "");
}
function parse(tokens) {
  var _a;
  const tree = [];
  let bpScope = { breakpoint: "-", values: [] };
  const getEmptyBucket = () => ({ sel: "", values: [] });
  let valueBucket = getEmptyBucket();
  for (const t of tokens) {
    if (!controlTokens.includes(t)) {
      valueBucket.values.push(t);
      continue;
    }
    if (t === "{") {
      bpScope = { breakpoint: valueBucket.values.join(","), values: [] };
      valueBucket = getEmptyBucket();
    }
    if (t === ";") {
      if (tree.length === 0) {
        tree.push(bpScope);
      }
      if (valueBucket.values.length > 0) {
        bpScope.values.push(valueBucket);
        valueBucket = getEmptyBucket();
      }
    }
    if (t === ":") {
      const sel = (_a = valueBucket.values.pop()) != null ? _a : "";
      valueBucket.sel = sel;
    }
    if (bpScope && t === "}") {
      bpScope.values.push(valueBucket);
      valueBucket = getEmptyBucket();
      tree.push(bpScope);
    }
  }
  if (valueBucket.values.length > 0) {
    bpScope.values.push(valueBucket);
  }
  if (tree.length === 0) {
    tree.push(bpScope);
  }
  return tree;
}
function parseBreakpoint(bp) {
  const bps = bp.split(",");
  return bps.map((b) => {
    const match = b.trim().match(breakpointValueRegex);
    if (!match) {
      throw new Error(`Invalid breakpoint specifier: ${b}`);
    }
    return { condition: match[1], value: match[2] };
  });
}
function toCss(selector, property2, tree, valueTemplate = (v) => v, styleTemplate = (p, vs) => `${p}: ${vs.join(" ")};`) {
  let css = "";
  for (const bp of tree) {
    const bpSpecs = bp.breakpoint === "-" ? [{ condition: ">=", value: "0" }] : parseBreakpoint(bp.breakpoint);
    const query = bpSpecs.map(
      (b) => {
        var _a, _b;
        return `(${((_a = b.condition) == null ? void 0 : _a.includes("<")) ? "max-width" : "min-width"}: ${(_b = viewportBreakpoints[b.value]) != null ? _b : b.value})`;
      }
    ).join(" and ");
    const mq = `@media ${query} {${bp.values.map((bpValues) => {
      let sel = selector;
      if (bpValues.sel.length > 0)
        sel = selector === ":host" ? `:host(:${bpValues.sel})` : `${selector}:${bpValues.sel}`;
      const style = styleTemplate(
        property2,
        bpValues.values.map(valueTemplate)
      );
      if (bpValues.sel === "hover")
        return `@media (hover: hover) {${sel}{${style}}}`;
      else
        return `${sel}{${style}}`;
    }).join("")}}`;
    css += mq;
  }
  return css;
}

// libs/core/src/utils/decorators/style-expression-property.ts
function styleExpressionProperty(options) {
  return (proto, descriptor) => {
    var _a, _b, _c;
    const sel = (_a = options == null ? void 0 : options.selector) != null ? _a : String(":host");
    const prop = (_b = options == null ? void 0 : options.property) != null ? _b : String(descriptor);
    const valueTemplate = (_c = options == null ? void 0 : options.valueTemplate) != null ? _c : (v) => `var(--gds-space-${v}, 0)`;
    const styleTemplate = options == null ? void 0 : options.styleTemplate;
    property({ attribute: options == null ? void 0 : options.attribute })(proto, descriptor);
    watch(descriptor)(proto, descriptor, {
      value: function(oldValue, newValue) {
        const ast = parse(tokenize(newValue));
        const css = toCss(
          sel,
          prop,
          ast,
          valueTemplate.bind(this),
          styleTemplate == null ? void 0 : styleTemplate.bind(this)
        );
        this[`__${String(descriptor)}_ast`] = ast;
        this._dynamicStylesController.inject(
          `sep_${String(descriptor)}`,
          unsafeCSS(css)
        );
      }
    });
  };
}

export {
  styleExpressionProperty
};
